@@ -5,6 +5,7 @@ package Pugs::Emitter::Rule::Perl5::Ratchet;
 
 use strict;
 use warnings;
+use Pugs::Emitter::Rule::Perl5::CharClass;
 use Data::Dumper;
 $Data::Dumper::Indent = 1;
 
@@ -748,14 +749,9 @@ sub constant {
     call_constant( @_ );
 }
 
-use vars qw( %char_class );
-BEGIN {
-    %char_class = map { $_ => 1 } qw( 
-        alpha alnum ascii blank
-        cntrl digit graph lower
-        print punct space upper
-        word  xdigit
-    );
+sub char_class {
+    my $cmd = Pugs::Emitter::Rule::Perl5::CharClass::emit( $_[0] );
+    return call_perl5($cmd, $_[1]);
 }
 
 sub metasyntax {
@@ -837,42 +833,12 @@ sub metasyntax {
         warn "<\"...\"> not implemented";
         return;
     }
-    if ( $prefix =~ /[-+[]/ ) {   # character class 
-        #die "SET ratchet: $cmd\n";
-        $cmd =~ s/\.\./-/g;
-        if ( $cmd =~ /^ - \s* \[ (.*) /x ) {
-           $cmd = '[^' . $1;
-        } 
-        elsif ( $cmd =~ /^ - \s* (.*) /x ) {
-           #$cmd = substr($cmd, 1);
-           my $name = $1;
-           $cmd = ( $name =~ /^is/ )
-                ? "\\P{$name}"
-                : "[^[:$name:]]";
-        } 
-        elsif ( $cmd =~ /^ \+ \s* \[ (.*) /x ) {
-           $cmd = '[' . $1;
-	    }
-        elsif ( $cmd =~ /^ \+ \s* (.*) /x ) {
-           my $name = $1;
-           $cmd = ( $name =~ /^is/ )
-                ? "\\p{$name}"
-                : "[[:$name:]]";
-        } 
-        $cmd =~ s/\s+|\n//g;
-        # XXX <[^a]> means [\^a] instead of [^a] in perl5re
-        return call_perl5($cmd, $_[1]);
-    }
     if ( $prefix eq '?' ) {   # non_capturing_subrule / code assertion
         $cmd = substr( $cmd, 1 );
         if ( $cmd =~ /^{/ ) {
             warn "code assertion not implemented";
             return;
         }
-        if ( exists $char_class{$cmd} ) {
-            # XXX - inlined char classes are not inheritable, but this should be ok
-            return call_perl5( "[[:$cmd:]]", $_[1] );
-        }
         my @param; # TODO
         my $subrule = $cmd;
         return
