@@ -7,6 +7,7 @@ use warnings;
 use Data::Dumper;
 #use Class::InsideOut qw( public register id );
 use Scalar::Util qw( refaddr blessed );
+use Pugs::Runtime::StrPos;
 
 use overload (
     '@{}'    => \&array,
@@ -36,11 +37,31 @@ sub DESTROY {
 }
 
 sub data  {    $_data{refaddr $_[0]}           }
-sub from  {  ${$_data{refaddr $_[0]}->{from}}  }
-sub to    {  ${$_data{refaddr $_[0]}->{to}}    }
 sub bool  {  ${$_data{refaddr $_[0]}->{bool}}  }
 sub array {    $_data{refaddr $_[0]}->{match}  }
 
+sub from  {  
+
+    return ${$_data{refaddr $_[0]}->{from}};
+
+    my $obj = $_data{refaddr $_[0]};
+    #return ${$obj->{from}} if blessed ${$obj->{from}};
+    Pugs::Runtime::StrPos->from_str_codes( ${$obj->{str}}, ${$obj->{from}} );
+}
+sub pos   { $_[0]->to } # pugs pos.t - lvalue ??? 
+sub to    {  
+
+    return ${$_data{refaddr $_[0]}->{to}};
+
+    my $obj = $_data{refaddr $_[0]};
+    #return ${$obj->{to}} if blessed ${$obj->{to}};
+    #print "TO: ",${$obj->{to}},"\n";
+    Pugs::Runtime::StrPos->from_str_codes( ${$obj->{str}}, ${$obj->{to}} );
+}
+# "low-level" position defaults to perl5-utf8
+sub from_as_codes  {  ${$_data{refaddr $_[0]}->{from}}  }
+sub to_as_codes    {  ${$_data{refaddr $_[0]}->{to}}    }
+
 sub hash  {   
     my $array = $_data{refaddr $_[0]}->{match};
     return {
@@ -95,10 +116,10 @@ sub perl {
 }
 
 sub yaml {
-    eval { use YAML::Syck };
+    require YAML::Syck;
     # interoperability with other YAML/Syck bindings:
     $YAML::Syck::ImplicitTyping = 1;
-    Dump( $_[0] );
+    YAML::Syck::Dump( $_[0] );
 }
 
 # for Pugs interoperability
@@ -200,11 +221,16 @@ If there is no capture, return the matched substring
 
 * from
 
-- return the string position where the match started
+- return the string position (a C<Pugs::Runtime::StrPos> object) where the match started.
 
 * to
 
-- return the string position immediately after where the match finished
+- return the string position (a C<Pugs::Runtime::StrPos> object) immediately after where the match finished.
+
+* from_as_codes
+* to_as_codes
+
+- same as C<from>/C<to> methods, but return perl5 integers.
 
 =head1 "Hash" methods
 
@@ -222,7 +248,7 @@ If there is no capture, return the matched substring
 
 =head1 OVERLOADS
 
-* $match->()
+* $$match
 
 - return the capture object
 
